@@ -18,34 +18,34 @@
 
 using namespace std;
 
-MagickDLLDecl int Magick::operator == ( const Magick::Coordinate& left_,
+MagickPPExport int Magick::operator == ( const Magick::Coordinate& left_,
                                         const Magick::Coordinate& right_ )
 {
   return ( ( left_.x() == right_.x() ) && ( left_.y() == right_.y() ) );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::Coordinate& left_,
+MagickPPExport int Magick::operator != ( const Magick::Coordinate& left_,
                                         const Magick::Coordinate& right_ )
 {
   return ( ! (left_ == right_) );
 }
-MagickDLLDecl int Magick::operator >  ( const Magick::Coordinate& left_,
+MagickPPExport int Magick::operator >  ( const Magick::Coordinate& left_,
                                         const Magick::Coordinate& right_ )
 {
   return ( !( left_ < right_ ) && ( left_ != right_ ) );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::Coordinate& left_,
+MagickPPExport int Magick::operator <  ( const Magick::Coordinate& left_,
                                         const Magick::Coordinate& right_ )
 {
   // Based on distance from origin
   return  ( (sqrt(left_.x()*left_.x() + left_.y()*left_.y())) <
             (sqrt(right_.x()*right_.x() + right_.y()*right_.y())) );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::Coordinate& left_,
+MagickPPExport int Magick::operator >= ( const Magick::Coordinate& left_,
                                         const Magick::Coordinate& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::Coordinate& left_,
+MagickPPExport int Magick::operator <= ( const Magick::Coordinate& left_,
                                         const Magick::Coordinate& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -100,32 +100,32 @@ void Magick::Drawable::operator()( MagickCore::DrawingWand * context_ ) const
     dp->operator()( context_ );
 }
 
-MagickDLLDecl int Magick::operator == ( const Magick::Drawable& /*left_*/,
+MagickPPExport int Magick::operator == ( const Magick::Drawable& /*left_*/,
                                         const Magick::Drawable& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::Drawable& /*left_*/,
+MagickPPExport int Magick::operator != ( const Magick::Drawable& /*left_*/,
                                         const Magick::Drawable& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator > ( const Magick::Drawable& /*left_*/,
+MagickPPExport int Magick::operator > ( const Magick::Drawable& /*left_*/,
                                        const Magick::Drawable& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::Drawable& /*left_*/,
+MagickPPExport int Magick::operator <  ( const Magick::Drawable& /*left_*/,
                                         const Magick::Drawable& /*right_*/ )
 {
   return  ( 0 );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::Drawable& left_,
+MagickPPExport int Magick::operator >= ( const Magick::Drawable& left_,
                                         const Magick::Drawable& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::Drawable& left_,
+MagickPPExport int Magick::operator <= ( const Magick::Drawable& left_,
                                         const Magick::Drawable& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -180,32 +180,32 @@ void Magick::VPath::operator()( MagickCore::DrawingWand * context_ ) const
     dp->operator()( context_ );
 }
 
-MagickDLLDecl int Magick::operator == ( const Magick::VPath& /*left_*/,
+MagickPPExport int Magick::operator == ( const Magick::VPath& /*left_*/,
                                         const Magick::VPath& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::VPath& /*left_*/,
+MagickPPExport int Magick::operator != ( const Magick::VPath& /*left_*/,
                                         const Magick::VPath& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator > ( const Magick::VPath& /*left_*/,
+MagickPPExport int Magick::operator > ( const Magick::VPath& /*left_*/,
                                        const Magick::VPath& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::VPath& /*left_*/,
+MagickPPExport int Magick::operator <  ( const Magick::VPath& /*left_*/,
                                         const Magick::VPath& /*right_*/ )
 {
   return  ( 0 );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::VPath& left_,
+MagickPPExport int Magick::operator >= ( const Magick::VPath& left_,
                                         const Magick::VPath& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::VPath& left_,
+MagickPPExport int Magick::operator <= ( const Magick::VPath& left_,
                                         const Magick::VPath& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -1042,7 +1042,7 @@ Magick::DrawableDashArray::DrawableDashArray
 }
 Magick::DrawableDashArray::~DrawableDashArray( void )
 {
-  delete _dasharray;
+  delete [] _dasharray;
   _size = 0;
   _dasharray = 0;
 }
@@ -1402,32 +1402,32 @@ Magick::DrawableBase* Magick::DrawableViewbox::copy() const
 //
 // PathArcArgs
 //
-MagickDLLDecl int Magick::operator == ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator == ( const Magick::PathArcArgs& /*left_*/,
                                         const Magick::PathArcArgs& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator != ( const Magick::PathArcArgs& /*left_*/,
                                         const Magick::PathArcArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator > ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator > ( const Magick::PathArcArgs& /*left_*/,
                                        const Magick::PathArcArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator <  ( const Magick::PathArcArgs& /*left_*/,
                                         const Magick::PathArcArgs& /*right_*/ )
 {
   return  ( false );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::PathArcArgs& left_,
+MagickPPExport int Magick::operator >= ( const Magick::PathArcArgs& left_,
                                         const Magick::PathArcArgs& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::PathArcArgs& left_,
+MagickPPExport int Magick::operator <= ( const Magick::PathArcArgs& left_,
                                         const Magick::PathArcArgs& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -1553,32 +1553,32 @@ Magick::VPathBase* Magick::PathClosePath::copy() const
 //
 // Path Curveto (Cubic Bezier)
 //
-MagickDLLDecl int Magick::operator == ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator == ( const Magick::PathCurvetoArgs& /*left_*/,
                                         const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator != ( const Magick::PathCurvetoArgs& /*left_*/,
                                         const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator > ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator > ( const Magick::PathCurvetoArgs& /*left_*/,
                                        const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator <  ( const Magick::PathCurvetoArgs& /*left_*/,
                                         const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return  ( false );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::PathCurvetoArgs& left_,
+MagickPPExport int Magick::operator >= ( const Magick::PathCurvetoArgs& left_,
                                         const Magick::PathCurvetoArgs& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::PathCurvetoArgs& left_,
+MagickPPExport int Magick::operator <= ( const Magick::PathCurvetoArgs& left_,
                                         const Magick::PathCurvetoArgs& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -1758,37 +1758,37 @@ Magick::VPathBase* Magick::PathSmoothCurvetoRel::copy() const
 //
 // Quadratic Curveto (Quadratic Bezier)
 //
-MagickDLLDecl int Magick::operator ==
+MagickPPExport int Magick::operator ==
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator !=
+MagickPPExport int Magick::operator !=
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator >
+MagickPPExport int Magick::operator >
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator < 
+MagickPPExport int Magick::operator < 
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return  ( 0 );
 }
-MagickDLLDecl int Magick::operator >=
+MagickPPExport int Magick::operator >=
 ( const Magick::PathQuadraticCurvetoArgs& left_,
   const Magick::PathQuadraticCurvetoArgs& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <=
+MagickPPExport int Magick::operator <=
 ( const Magick::PathQuadraticCurvetoArgs& left_,
   const Magick::PathQuadraticCurvetoArgs& right_ )
 {
