@@ -3,6 +3,11 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%        CCCC  L      IIIII  PPPP   BBBB    OOO    AAA   RRRR   DDDD          %
+%       C      L        I    P   P  B   B  O   O  A   A  R   R  D   D         %
+%       C      L        I    PPP    BBBB   O   O  AAAAA  RRRR   D   D         %
+%       C      L        I    P      B   B  O   O  A   A  R R    D   D         %
+%        CCCC  LLLLL  IIIII  P      BBBB    OOO   A   A  R  R   DDDD          %
 %                                                                             %
 %                                                                             %
 %                        Read/Write Windows Clipboard.                        %
@@ -12,7 +17,7 @@
 %                                 May 2002                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -101,15 +106,15 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
   Image
     *image;
 
-  ssize_t
-    y;
-
   register ssize_t
     x;
 
   register PixelPacket
     *q;
 
+  ssize_t
+    y;
+
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
@@ -208,10 +213,10 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
           break;
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          q->red=ScaleCharToQuantum(pBits->rgbRed);
-          q->green=ScaleCharToQuantum(pBits->rgbGreen);
-          q->blue=ScaleCharToQuantum(pBits->rgbBlue);
-          SetOpacityPixelComponent(q,OpaqueOpacity);
+          SetPixelRed(q,ScaleCharToQuantum(pBits->rgbRed));
+          SetPixelGreen(q,ScaleCharToQuantum(pBits->rgbGreen));
+          SetPixelBlue(q,ScaleCharToQuantum(pBits->rgbBlue));
+          SetPixelOpacity(q,OpaqueOpacity);
           pBits++;
           q++;
         }
@@ -261,7 +266,7 @@ ModuleExport size_t RegisterCLIPBOARDImage(void)
   entry->encoder=(EncodeImageHandler *) WriteCLIPBOARDImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->format_type=ExplicitFormatType;
+  entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("The system clipboard");
   entry->module=ConstantString("CLIPBOARD");
   (void) RegisterMagickInfo(entry);
@@ -334,7 +339,7 @@ static MagickBooleanType WriteCLIPBOARDImage(const ImageInfo *image_info,
     HBITMAP
       bitmapH;
 
-    OpenClipboard( NULL );
+    OpenClipboard(NULL);
     EmptyClipboard();
     bitmapH=(HBITMAP) ImageToHBITMAP(image);
     SetClipboardData(CF_BITMAP,bitmapH);
