@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -102,9 +102,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
     *canvas_image,
     *image;
 
-  ssize_t
-    y;
-
   MagickBooleanType
     status;
 
@@ -120,12 +117,13 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   register ssize_t
     i;
 
-  ssize_t
-    count;
-
   size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -142,7 +140,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
-  image->colorspace=CMYKColorspace;
+  SetImageColorspace(image,CMYKColorspace);
   if (image_info->interlace != PartitionInterlace)
     {
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
@@ -151,13 +149,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           image=DestroyImageList(image);
           return((Image *) NULL);
         }
-      for (i=0; i < image->offset; i++)
-        if (ReadBlobByte(image) == EOF)
-          {
-            ThrowFileException(exception,CorruptImageError,
-              "UnexpectedEndOfFile",image->filename);
-            break;
-          }
+      if (DiscardBlobBytes(image,image->offset) == MagickFalse)
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
     }
   /*
     Create virtual canvas to support cropping (i.e. image.cmyk[100x100+10+20]).
@@ -201,7 +195,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    image->colorspace=CMYKColorspace;
+    SetImageColorspace(image,CMYKColorspace);
     switch (image_info->interlace)
     {
       case NoInterlace:
@@ -226,12 +220,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register IndexPacket
             *restrict indexes;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -260,13 +254,14 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               indexes=GetAuthenticIndexQueue(image);
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetRedPixelComponent(q,GetRedPixelComponent(p));
-                SetGreenPixelComponent(q,GetGreenPixelComponent(p));
-                SetBluePixelComponent(q,GetBluePixelComponent(p));
-                indexes[x]=canvas_indexes[image->extract_info.x+x];
-                SetOpacityPixelComponent(q,OpaqueOpacity);
+                SetPixelRed(q,GetPixelRed(p));
+                SetPixelGreen(q,GetPixelGreen(p));
+                SetPixelBlue(q,GetPixelBlue(p));
+                SetPixelBlack(indexes+x,GetPixelBlack(
+                  canvas_indexes+image->extract_info.x+x));
+                SetPixelOpacity(q,OpaqueOpacity);
                 if (image->matte != MagickFalse)
-                  SetOpacityPixelComponent(q,GetOpacityPixelComponent(p));
+                  SetPixelOpacity(q,GetPixelOpacity(p));
                 p++;
                 q++;
               }
@@ -315,12 +310,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register IndexPacket
             *restrict indexes;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -354,13 +349,34 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 {
                   switch (quantum_type)
                   {
-                    case CyanQuantum: SetRedPixelComponent(q,GetRedPixelComponent(p)); break;
-                    case MagentaQuantum: SetGreenPixelComponent(q,GetGreenPixelComponent(p)); break;
-                    case YellowQuantum: SetBluePixelComponent(q,GetBluePixelComponent(p)); break;
-                    case BlackQuantum: indexes[x]=
-                      canvas_indexes[image->extract_info.x+x];; break;
-                    case OpacityQuantum: SetOpacityPixelComponent(q,GetOpacityPixelComponent(p)); break;
-                    default: break;
+                    case CyanQuantum:
+                    {
+                      SetPixelCyan(q,GetPixelCyan(p));
+                      break;
+                    }
+                    case MagentaQuantum:
+                    {
+                      SetPixelMagenta(q,GetPixelMagenta(p));
+                      break;
+                    }
+                    case YellowQuantum:
+                    {
+                      SetPixelYellow(q,GetPixelYellow(p));
+                      break;
+                    }
+                    case BlackQuantum:
+                    {
+                      SetPixelIndex(indexes+x,GetPixelIndex(
+                        canvas_indexes+image->extract_info.x+x));
+                      break;
+                    }
+                    case OpacityQuantum:
+                    {
+                      SetPixelOpacity(q,GetPixelOpacity(p));
+                      break;
+                    }
+                    default:
+                      break;
                   }
                   p++;
                   q++;
@@ -395,12 +411,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register const PixelPacket
             *restrict p;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -427,7 +443,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetRedPixelComponent(q,GetRedPixelComponent(p));
+                SetPixelRed(q,GetPixelRed(p));
                 p++;
                 q++;
               }
@@ -447,12 +463,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register const PixelPacket
             *restrict p;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -479,7 +495,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetGreenPixelComponent(q,GetGreenPixelComponent(p));
+                SetPixelGreen(q,GetPixelGreen(p));
                 p++;
                 q++;
               }
@@ -499,12 +515,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register const PixelPacket
             *restrict p;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -531,7 +547,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetBluePixelComponent(q,GetBluePixelComponent(p));
+                SetPixelBlue(q,GetPixelBlue(p));
                 p++;
                 q++;
               }
@@ -557,12 +573,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register IndexPacket
             *restrict indexes;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -591,7 +607,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               indexes=GetAuthenticIndexQueue(image);
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                indexes[x]=canvas_indexes[image->extract_info.x+x];
+                SetPixelIndex(indexes+x,GetPixelIndex(
+                  canvas_indexes+image->extract_info.x+x));
                 p++;
                 q++;
               }
@@ -613,12 +630,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               register const PixelPacket
                 *restrict p;
 
-              register ssize_t
-                x;
-
               register PixelPacket
                 *restrict q;
 
+              register ssize_t
+                x;
+
               if (count != (ssize_t) length)
                 {
                   ThrowFileException(exception,CorruptImageError,
@@ -646,7 +663,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                     break;
                   for (x=0; x < (ssize_t) image->columns; x++)
                   {
-                    SetOpacityPixelComponent(q,GetOpacityPixelComponent(p));
+                    SetPixelOpacity(q,GetPixelOpacity(p));
                     p++;
                     q++;
                   }
@@ -683,13 +700,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
-        for (i=0; i < image->offset; i++)
-          if (ReadBlobByte(image) == EOF)
-            {
-              ThrowFileException(exception,CorruptImageError,
-                "UnexpectedEndOfFile",image->filename);
-              break;
-            }
+        if (DiscardBlobBytes(image,image->offset) == MagickFalse)
+          ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+            image->filename);
         length=GetQuantumExtent(canvas_image,quantum_info,CyanQuantum);
         for (i=0; i < (ssize_t) scene; i++)
           for (y=0; y < (ssize_t) image->extract_info.height; y++)
@@ -705,12 +718,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register const PixelPacket
             *restrict p;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -737,7 +750,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetRedPixelComponent(q,GetRedPixelComponent(p));
+                SetPixelRed(q,GetPixelRed(p));
                 p++;
                 q++;
               }
@@ -776,12 +789,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register const PixelPacket
             *restrict p;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -808,7 +821,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetGreenPixelComponent(q,GetGreenPixelComponent(p));
+                SetPixelGreen(q,GetPixelGreen(p));
                 p++;
                 q++;
               }
@@ -847,12 +860,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register const PixelPacket
             *restrict p;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -879,7 +892,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetBluePixelComponent(q,GetBluePixelComponent(p));
+                SetPixelBlue(q,GetPixelBlue(p));
                 p++;
                 q++;
               }
@@ -924,12 +937,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           register IndexPacket
             *restrict indexes;
 
-          register ssize_t
-            x;
-
           register PixelPacket
             *restrict q;
 
+          register ssize_t
+            x;
+
           if (count != (ssize_t) length)
             {
               ThrowFileException(exception,CorruptImageError,
@@ -958,7 +971,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               indexes=GetAuthenticIndexQueue(image);
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                indexes[x]=canvas_indexes[image->extract_info.x+x];
+                SetPixelIndex(indexes+x,GetPixelIndex(
+                  canvas_indexes+image->extract_info.x+x));
                 p++;
                 q++;
               }
@@ -999,12 +1013,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               register const PixelPacket
                 *restrict p;
 
-              register ssize_t
-                x;
-
               register PixelPacket
                 *restrict q;
 
+              register ssize_t
+                x;
+
               if (count != (ssize_t) length)
                 {
                   ThrowFileException(exception,CorruptImageError,
@@ -1031,7 +1045,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                     break;
                   for (x=0; x < (ssize_t) image->columns; x++)
                   {
-                    SetOpacityPixelComponent(q,GetOpacityPixelComponent(p));
+                    SetPixelOpacity(q,GetPixelOpacity(p));
                     p++;
                     q++;
                   }
@@ -1122,7 +1136,6 @@ ModuleExport size_t RegisterCMYKImage(void)
   entry->encoder=(EncodeImageHandler *) WriteCMYKImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw cyan, magenta, yellow, and black "
     "samples");
   entry->module=ConstantString("CMYK");
@@ -1132,7 +1145,6 @@ ModuleExport size_t RegisterCMYKImage(void)
   entry->encoder=(EncodeImageHandler *) WriteCMYKImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw cyan, magenta, yellow, black, and "
     "alpha samples");
   entry->module=ConstantString("CMYK");
@@ -1194,9 +1206,6 @@ ModuleExport void UnregisterCMYKImage(void)
 static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
   Image *image)
 {
-  ssize_t
-    y;
-
   MagickBooleanType
     status;
 
@@ -1210,7 +1219,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
     quantum_type;
 
   ssize_t
-    count;
+    count,
+    y;
 
   size_t
     length;
@@ -1273,8 +1283,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            quantum_type,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,quantum_type,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1301,23 +1311,23 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            CyanQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,CyanQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            MagentaQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,MagentaQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            YellowQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,YellowQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            BlackQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,BlackQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1352,8 +1362,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            CyanQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,CyanQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1372,8 +1382,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            MagentaQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,MagentaQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1392,8 +1402,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            YellowQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,YellowQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1412,8 +1422,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            BlackQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,BlackQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1431,8 +1441,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
               register const PixelPacket
                 *restrict p;
 
-              p=GetVirtualPixels(image,0,y,image->columns,1,
-                &image->exception);
+              p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
               length=ExportQuantumPixels(image,(const CacheView *) NULL,
@@ -1477,8 +1486,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            CyanQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,CyanQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1503,8 +1512,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            MagentaQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,MagentaQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1529,8 +1538,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            YellowQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,YellowQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
@@ -1555,8 +1564,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            BlackQuantum,pixels,&image->exception);
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,BlackQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
