@@ -53,6 +53,8 @@
 #include "magick/cache.h"
 #include "magick/color-private.h"
 #include "magick/colormap.h"
+#include "magick/colorspace-private.h"
+#include "magick/distort.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -64,12 +66,13 @@
 #include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/option.h"
+#include "magick/pixel.h"
 #include "magick/resource_.h"
-#include "magick/shear.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/module.h"
 #include "magick/transform.h"
+#include "magick/utility-private.h"
 #if defined(MAGICKCORE_ZLIB_DELEGATE)
  #include "zlib.h"
 #endif
@@ -189,34 +192,46 @@ static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVa
     MaxVal = 1;
 
   exception=(&image->exception);
-  q = QueueAuthenticPixels(image, 0, y, image->columns, 1,exception);
+  q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
   if (q == (PixelPacket *) NULL)
     return;
   for (x = 0; x < (ssize_t) image->columns; x++)
   {
     if (*p > 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->red);
-      if (f + q->red > QuantumRange)
-        q->red = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetPixelRed(q));
+      if (f + GetPixelRed(q) > QuantumRange)
+        SetPixelRed(q,QuantumRange);
       else
-        q->red += (int) f;
-      if ((int) f / 2.0 > q->green)
-        q->green = q->blue = 0;
+        SetPixelRed(q,GetPixelRed(q)+(int) f);
+      if ((int) f / 2.0 > GetPixelGreen(q))
+        {
+          SetPixelGreen(q,0);
+          SetPixelBlue(q,0);
+        }
       else
-        q->green = q->blue -= (int) (f / 2.0);
+        {
+          SetPixelBlue(q,GetPixelBlue(q)-(int) (f/2.0));
+          SetPixelGreen(q,GetPixelBlue(q));
+        }
     }
     if (*p < 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->blue);
-      if (f + q->blue > QuantumRange)
-        q->blue = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetPixelBlue(q));
+      if (f + GetPixelBlue(q) > QuantumRange)
+        SetPixelBlue(q,QuantumRange);
       else
-        q->blue += (int) f;
+        SetPixelBlue(q,GetPixelBlue(q)+(int) f);
       if ((int) f / 2.0 > q->green)
-        q->green = q->red = 0;
+        {
+          SetPixelRed(q,0);
+          SetPixelGreen(q,0);
+        }
       else
-        q->green = q->red -= (int) (f / 2.0);
+        {
+          SetPixelRed(q,GetPixelRed(q)-(int) (f/2.0));
+          SetPixelGreen(q,GetPixelRed(q));
+        }
     }
     p++;
     q++;
@@ -250,27 +265,39 @@ static void InsertComplexFloatRow(float *p, int y, Image * image, double MinVal,
   {
     if (*p > 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->red);
-      if (f + q->red > QuantumRange)
-        q->red = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetPixelRed(q));
+      if (f + GetPixelRed(q) > QuantumRange)
+        SetPixelRed(q,QuantumRange);
       else
-        q->red += (int) f;
-      if ((int) f / 2.0 > q->green)
-        q->green = q->blue = 0;
+        SetPixelRed(q,GetPixelRed(q)+(int) f);
+      if ((int) f / 2.0 > GetPixelGreen(q))
+        {
+          SetPixelGreen(q,0);
+          SetPixelBlue(q,0);
+        }
       else
-        q->green = q->blue -= (int) (f / 2.0);
+        {
+          SetPixelBlue(q,GetPixelBlue(q)-(int) (f/2.0));
+          SetPixelGreen(q,GetPixelBlue(q));
+        }
     }
     if (*p < 0)
     {
-      f = (*p / MaxVal) * (QuantumRange - q->blue);
-      if (f + q->blue > QuantumRange)
-        q->blue = QuantumRange;
+      f = (*p / MaxVal) * (QuantumRange - GetPixelBlue(q));
+      if (f + GetPixelBlue(q) > QuantumRange)
+        SetPixelBlue(q,QuantumRange);
       else
-        q->blue += (int) f;
+        SetPixelBlue(q,GetPixelBlue(q)+(int) f);
       if ((int) f / 2.0 > q->green)
-        q->green = q->red = 0;
+        {
+          SetPixelGreen(q,0);
+          SetPixelRed(q,0);
+        }
       else
-        q->green = q->red -= (int) (f / 2.0);
+        {
+          SetPixelRed(q,GetPixelRed(q)-(int) (f/2.0));
+          SetPixelGreen(q,GetPixelRed(q));
+        }
     }
     p++;
     q++;
@@ -399,9 +426,9 @@ static void FixSignedValues(PixelPacket *q, int y)
      /* Please note that negative values will overflow
         Q=8; QuantumRange=255: <0;127> + 127+1 = <128; 255> 
            <-1;-128> + 127+1 = <0; 127> */
-    q->red += QuantumRange/2 + 1;
-    q->green += QuantumRange/ + 1;
-    q->blue += QuantumRange/ + 1;
+    SetPixelRed(q,GetPixelRed(q)+QuantumRange/2+1);
+    SetPixelGreen(q,GetPixelGreen(q)+QuantumRange/2+1);
+    SetPixelBlue(q,GetPixelBlue(q)+QuantumRange/2+1);
     q++;
   }
 }
@@ -446,10 +473,10 @@ static void RelinquishZIPMemory(voidpf context,voidpf memory)
 }
 #endif
 
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
 /** This procedure decompreses an image block for a new MATLAB format. */
 static Image *DecompressBlock(Image *orig, MagickOffsetType Size, ImageInfo *clone_info, ExceptionInfo *exception)
 {
-#if defined(MAGICKCORE_ZLIB_DELEGATE)
 
 Image *image2;
 void *CacheBlock, *DecompressBlock;
@@ -465,7 +492,7 @@ int status;
   {
     fclose(clone_info->file);
     clone_info->file = NULL;
-    (void) unlink(clone_info->filename);
+    (void) remove_utf8(clone_info->filename);
   }
 
   CacheBlock = AcquireQuantumMemory((size_t)((Size<16384)?Size:16384),sizeof(unsigned char *));
@@ -506,6 +533,7 @@ int status;
       zip_info.next_out = (Bytef *) DecompressBlock;
       status = inflate(&zip_info,Z_NO_FLUSH);      
       extent=fwrite(DecompressBlock, 4096-zip_info.avail_out, 1, mat_file);
+      (void) extent;
 
       if(status == Z_STREAM_END) goto DblBreak;
     }
@@ -528,19 +556,13 @@ EraseFile:
     fclose(clone_info->file);
     clone_info->file = NULL;
 UnlinkFile:
-    (void) unlink(clone_info->filename);
+    (void) remove_utf8(clone_info->filename);
     return NULL; 
   }
 
   return image2;
-#else
-  (void) orig;
-  (void) Size;
-  (void) clone_info;
-  (void) exception;
-  return NULL;
-#endif
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -597,7 +619,8 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char *BImgBuff = NULL;
   double MinVal, MaxVal;
   size_t Unknown6;
-  unsigned z;
+  unsigned z, z2;
+  unsigned Frames;
   int logging;
   int sample_size;
   MagickOffsetType filepos=0x80;
@@ -664,6 +687,7 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   filepos = TellBlob(image);
   while(!EOFBlob(image)) /* object parser loop */
   {
+    Frames = 1;
     (void) SeekBlob(image,filepos,SEEK_SET);
     /* printf("pos=%X\n",TellBlob(image)); */
 
@@ -703,11 +727,17 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
 
     switch(MATLAB_HDR.DimFlag)
     {     
-      case  8: z=1; break;      /* 2D matrix*/
-      case 12: z = ReadBlobXXXLong(image2);  /* 3D matrix RGB*/
+      case  8: z2=z=1; break;      /* 2D matrix*/
+      case 12: z2=z = ReadBlobXXXLong(image2);  /* 3D matrix RGB*/
            Unknown6 = ReadBlobXXXLong(image2);
+           (void) Unknown6;
          if(z!=3) ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
          break;
+      case 16: z2=z = ReadBlobXXXLong(image2);  /* 4D matrix animation */
+         if(z!=3 && z!=1)
+           ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
+          Frames = ReadBlobXXXLong(image2);
+         break;
       default: ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
     }  
 
@@ -753,11 +783,7 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   
     (void) ReadBlob(image2, 4, (unsigned char *) &size);     /* data size */
 
-      /* Image is gray when no complex flag is set and 2D Matrix */
-    if ((MATLAB_HDR.DimFlag == 8) &&
-        ((MATLAB_HDR.StructureFlag & FLAG_COMPLEX) == 0))
-      image->type=GrayscaleType;
-
+NEXT_FRAME:
     switch (CellType)
     {
       case miINT8:
@@ -810,6 +836,7 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       default:
         ThrowReaderException(CoderError, "UnsupportedCellTypeInTheMatrix");
     }
+    (void) sample_size;
     image->columns = MATLAB_HDR.SizeX;
     image->rows = MATLAB_HDR.SizeY;    
     quantum_info=AcquireQuantumInfo(clone_info,image);
@@ -819,6 +846,14 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     image->colors = one << image->depth;
     if (image->columns == 0 || image->rows == 0)
       goto MATLAB_KO;
+      /* Image is gray when no complex flag is set and 2D Matrix */
+    if ((MATLAB_HDR.DimFlag == 8) &&
+        ((MATLAB_HDR.StructureFlag & FLAG_COMPLEX) == 0))
+      {
+        SetImageColorspace(image,GRAYColorspace);
+        image->type=GrayscaleType;
+      }
+
 
     /* ----- Create gray palette ----- */
 
@@ -972,7 +1007,7 @@ done_reading:
     {
             fclose(clone_info->file);
             clone_info->file = NULL;
-            (void) unlink(clone_info->filename);
+            (void) remove_utf8(clone_info->filename);
     }
         }    
       }
@@ -984,11 +1019,34 @@ done_reading:
     image->columns=image->rows=0;
     image->colors=0;    
 
-      /* row scan buffer is no ssize_ter needed */
+      /* row scan buffer is no longer needed */
     RelinquishMagickMemory(BImgBuff);
     BImgBuff = NULL;
+
+    if(--Frames>0)
+    {
+      z = z2;
+      if(image2==NULL) image2 = image;
+      goto NEXT_FRAME;
+    }
+
+    if(image2!=NULL)
+      if(image2!=image)   /* Does shadow temporary decompressed image exist? */
+      {
+/*  CloseBlob(image2); */
+        DeleteImageFromList(&image2);
+        if(clone_info)
+        {
+          if(clone_info->file)
+          {
+            fclose(clone_info->file);
+            clone_info->file = NULL;
+            (void) unlink(clone_info->filename);
+          }
+         }
+       }
   }
-    clone_info=DestroyImageInfo(clone_info);
+  clone_info=DestroyImageInfo(clone_info);
 
   RelinquishMagickMemory(BImgBuff);
   CloseBlob(image);
@@ -1028,7 +1086,7 @@ done_reading:
     {
       fclose(clone_info->file);
       clone_info->file = NULL;
-      (void) unlink(clone_info->filename);
+      (void) remove_utf8(clone_info->filename);
     }
     DestroyImageInfo(clone_info);
     clone_info = NULL;
@@ -1072,7 +1130,7 @@ ModuleExport size_t RegisterMATImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMATImage;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("MATLAB image format");
+  entry->description=AcquireString("MATLAB level 5 image format");
   entry->module=AcquireString("MAT");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -1163,6 +1221,7 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter MAT");
+  (void) logging;
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(MagickFalse);
@@ -1175,7 +1234,7 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   (void) memcpy(&local_time,localtime(&current_time),sizeof(local_time));
 #endif
   (void) memset(MATLAB_HDR,' ',MagickMin(sizeof(MATLAB_HDR),124));
-  FormatMagickString(MATLAB_HDR,MaxTextExtent,"MATLAB 5.0 MAT-file, Platform: %s, Created on: %s %s %2d %2d:%2d:%2d %d",
+  FormatLocaleString(MATLAB_HDR,MaxTextExtent,"MATLAB 5.0 MAT-file, Platform: %s, Created on: %s %s %2d %2d:%2d:%2d %d",
     OsDesc,DayOfWTab[local_time.tm_wday],MonthsTab[local_time.tm_mon],
     local_time.tm_mday,local_time.tm_hour,local_time.tm_min,
     local_time.tm_sec,local_time.tm_year+1900);
@@ -1187,9 +1246,8 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
-    if (image->colorspace != RGBColorspace)
-      (void) TransformImageColorspace(image,RGBColorspace);
-
+    if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace);
     is_gray = IsGrayImage(image,&image->exception);
     z = is_gray ? 0 : 3;
 
