@@ -13,11 +13,11 @@
 %                       MagickCore Acceleration Methods                       %
 %                                                                             %
 %                              Software Design                                %
-%                              Anthony Thyssen                                %
+%                               John Cristy                                   %
 %                               January 2010                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,7 +33,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-% Morpology is the the application of various kernals, of any size and even
+% Morphology is the the application of various kernals, of any size and even
 % shape, to a image in various ways (typically binary, but not always).
 %
 % Convolution (weighted sum or average) is just one specific type of
@@ -52,6 +52,8 @@
 #include "magick/studio.h"
 #include "magick/accelerate.h"
 #include "magick/artifact.h"
+#include "magick/cache.h"
+#include "magick/cache-private.h"
 #include "magick/cache-view.h"
 #include "magick/color-private.h"
 #include "magick/enhance.h"
@@ -66,7 +68,6 @@
 #include "magick/monitor-private.h"
 #include "magick/accelerate.h"
 #include "magick/option.h"
-#include "magick/pixel-private.h"
 #include "magick/prepress.h"
 #include "magick/quantize.h"
 #include "magick/registry.h"
@@ -130,8 +131,8 @@
 #define CLOptions "-DCLQuantum=uint -DCLPixelType=uint4 " \
   "-DQuantumRange=%g -DMagickEpsilon=%g"
 #define CLPixelPacket  cl_uint4
-#elif (MAGICKCORE_QUANTUM_DEPTH == 32)
-#define CLOptions "-DCLQuantum=ulong -DCLPixelType=ulong4 " \
+#elif (MAGICKCORE_QUANTUM_DEPTH == 64)
+#define CLOptions "-DCLQuantum=ussize_t -DCLPixelType=ussize_t4 " \
   "-DQuantumRange=%g -DMagickEpsilon=%g"
 #define CLPixelPacket  cl_ulong4
 #endif
@@ -162,7 +163,7 @@ typedef struct _ConvolveInfo
     width,
     height;
 
-  cl_bool
+  cl_uint
     matte;
 
   cl_mem
@@ -171,7 +172,7 @@ typedef struct _ConvolveInfo
 
 static char
   *ConvolveKernel =
-    "static inline long ClampToCanvas(const long offset,const ulong range)\n"
+    "static inline long ClampToCanvas(const long offset,const unsigned long range)\n"
     "{\n"
     "  if (offset < 0L)\n"
     "    return(0L);\n"
@@ -180,35 +181,35 @@ static char
     "  return(offset);\n"
     "}\n"
     "\n"
-    "static inline CLQuantum ClampToQuantum(const double value)\n"
+    "static inline CLQuantum ClampToQuantum(const float value)\n"
     "{\n"
     "#if defined(MAGICKCORE_HDRI_SUPPORT)\n"
-    "  return((CLQuantum) value)\n"
+    "  return((CLQuantum) value);\n"
     "#else\n"
     "  if (value < 0.0)\n"
     "    return((CLQuantum) 0);\n"
-    "  if (value >= (double) QuantumRange)\n"
+    "  if (value >= (float) QuantumRange)\n"
     "    return((CLQuantum) QuantumRange);\n"
     "  return((CLQuantum) (value+0.5));\n"
     "#endif\n"
     "}\n"
     "\n"
     "__kernel void Convolve(const __global CLPixelType *input,\n"
-    "  __constant double *filter,const ulong width,const ulong height,\n"
-    "  const bool matte,__global CLPixelType *output)\n"
+    "  __constant float *filter,const unsigned long width,const unsigned long height,\n"
+    "  const unsigned int matte,__global CLPixelType *output)\n"
     "{\n"
-    "  const ulong columns = get_global_size(0);\n"
-    "  const ulong rows = get_global_size(1);\n"
+    "  const unsigned long columns = get_global_size(0);\n"
+    "  const unsigned long rows = get_global_size(1);\n"
     "\n"
     "  const long x = get_global_id(0);\n"
     "  const long y = get_global_id(1);\n"
     "\n"
-    "  const double scale = (1.0/QuantumRange);\n"
+    "  const float scale = (1.0/QuantumRange);\n"
     "  const long mid_width = (width-1)/2;\n"
     "  const long mid_height = (height-1)/2;\n"
-    "  double4 sum = { 0.0, 0.0, 0.0, 0.0 };\n"
-    "  double gamma = 0.0;\n"
-    "  register ulong i = 0;\n"
+    "  float4 sum = { 0.0, 0.0, 0.0, 0.0 };\n"
+    "  float gamma = 0.0;\n"
+    "  register unsigned long i = 0;\n"
     "\n"
     "  int method = 0;\n"
     "  if (matte != false)\n"
@@ -245,9 +246,9 @@ static char
     "      {\n"
     "        for (long u=(-mid_width); u <= mid_width; u++)\n"
     "        {\n"
-    "          const ulong index=ClampToCanvas(y+v,rows)*columns+\n"
+    "          const unsigned long index=ClampToCanvas(y+v,rows)*columns+\n"
     "            ClampToCanvas(x+u,columns);\n"
-    "          const double alpha=scale*(QuantumRange-input[index].w);\n"
+    "          const float alpha=scale*input[index].w;\n"
     "          sum.x+=alpha*filter[i]*input[index].x;\n"
     "          sum.y+=alpha*filter[i]*input[index].y;\n"
     "          sum.z+=alpha*filter[i]*input[index].z;\n"
@@ -264,7 +265,7 @@ static char
     "      {\n"
     "        for (long u=(-mid_width); u <= mid_width; u++)\n"
     "        {\n"
-    "          const ulong index=(y+v)*columns+(x+u);\n"
+    "          const unsigned long index=(y+v)*columns+(x+u);\n"
     "          sum.x+=filter[i]*input[index].x;\n"
     "          sum.y+=filter[i]*input[index].y;\n"
     "          sum.z+=filter[i]*input[index].z;\n"
@@ -280,8 +281,8 @@ static char
     "      {\n"
     "        for (long u=(-mid_width); u <= mid_width; u++)\n"
     "        {\n"
-    "          const ulong index=(y+v)*columns+(x+u);\n"
-    "          const double alpha=scale*(QuantumRange-input[index].w);\n"
+    "          const unsigned long index=(y+v)*columns+(x+u);\n"
+    "          const float alpha=scale*input[index].w;\n"
     "          sum.x+=alpha*filter[i]*input[index].x;\n"
     "          sum.y+=alpha*filter[i]*input[index].y;\n"
     "          sum.z+=alpha*filter[i]*input[index].z;\n"
@@ -293,8 +294,8 @@ static char
     "      break;\n"
     "    }\n"
     "  }\n"
-    "  gamma=1.0/(fabs(gamma) <= MagickEpsilon ? 1.0 : gamma);\n"
-    "  const ulong index = y*columns+x;\n"
+    "  gamma=MagickEpsilonReciprocal(gamma);\n"
+    "  const unsigned long index = y*columns+x;\n"
     "  output[index].x=ClampToQuantum(gamma*sum.x);\n"
     "  output[index].y=ClampToQuantum(gamma*sum.y);\n"
     "  output[index].z=ClampToQuantum(gamma*sum.z);\n"
@@ -318,13 +319,13 @@ static void ConvolveNotify(const char *message,const void *data,size_t length,
 }
 
 static MagickBooleanType BindConvolveParameters(ConvolveInfo *convolve_info,
-  const Image *image,const void *pixels,double *filter,
-  const unsigned long width,const unsigned long height,void *convolve_pixels)
+  const Image *image,const void *pixels,float *filter,const size_t width,
+  const size_t height,void *convolve_pixels)
 {
   cl_int
     status;
 
-  register int
+  register cl_uint
     i;
 
   size_t
@@ -334,19 +335,21 @@ static MagickBooleanType BindConvolveParameters(ConvolveInfo *convolve_info,
     Allocate OpenCL buffers.
   */
   length=image->columns*image->rows;
-  convolve_info->pixels=clCreateBuffer(convolve_info->context,CL_MEM_READ_ONLY |
-    CL_MEM_USE_HOST_PTR,length*sizeof(CLPixelPacket),(void *) pixels,&status);
+  convolve_info->pixels=clCreateBuffer(convolve_info->context,(cl_mem_flags)
+    (CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR),length*sizeof(CLPixelPacket),
+    (void *) pixels,&status);
   if ((convolve_info->pixels == (cl_mem) NULL) || (status != CL_SUCCESS))
     return(MagickFalse);
   length=width*height;
-  convolve_info->filter=clCreateBuffer(convolve_info->context,CL_MEM_READ_ONLY |
-    CL_MEM_USE_HOST_PTR,length*sizeof(cl_double),filter,&status);
+  convolve_info->filter=clCreateBuffer(convolve_info->context,(cl_mem_flags)
+    (CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR),length*sizeof(cl_float),filter,
+    &status);
   if ((convolve_info->filter == (cl_mem) NULL) || (status != CL_SUCCESS))
     return(MagickFalse);
   length=image->columns*image->rows;
   convolve_info->convolve_pixels=clCreateBuffer(convolve_info->context,
-    CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR,length*sizeof(CLPixelPacket),
-    convolve_pixels,&status);
+    (cl_mem_flags) (CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR),length*
+    sizeof(CLPixelPacket),convolve_pixels,&status);
   if ((convolve_info->convolve_pixels == (cl_mem) NULL) ||
       (status != CL_SUCCESS))
     return(MagickFalse);
@@ -372,8 +375,8 @@ static MagickBooleanType BindConvolveParameters(ConvolveInfo *convolve_info,
     &convolve_info->height);
   if (status != CL_SUCCESS)
     return(MagickFalse);
-  convolve_info->matte=(cl_bool) image->matte;
-  status=clSetKernelArg(convolve_info->kernel,i++,sizeof(cl_bool),(void *)
+  convolve_info->matte=(cl_uint) image->matte;
+  status=clSetKernelArg(convolve_info->kernel,i++,sizeof(cl_uint),(void *)
     &convolve_info->matte);
   if (status != CL_SUCCESS)
     return(MagickFalse);
@@ -392,12 +395,14 @@ static void DestroyConvolveBuffers(ConvolveInfo *convolve_info)
   cl_int
     status;
 
+  status=0;
   if (convolve_info->convolve_pixels != (cl_mem) NULL)
     status=clReleaseMemObject(convolve_info->convolve_pixels);
   if (convolve_info->pixels != (cl_mem) NULL)
     status=clReleaseMemObject(convolve_info->pixels);
   if (convolve_info->filter != (cl_mem) NULL)
     status=clReleaseMemObject(convolve_info->filter);
+  (void) status;
 }
 
 static ConvolveInfo *DestroyConvolveInfo(ConvolveInfo *convolve_info)
@@ -405,6 +410,7 @@ static ConvolveInfo *DestroyConvolveInfo(ConvolveInfo *convolve_info)
   cl_int
     status;
 
+  status=0;
   if (convolve_info->kernel != (cl_kernel) NULL)
     status=clReleaseKernel(convolve_info->kernel);
   if (convolve_info->program != (cl_program) NULL)
@@ -413,13 +419,14 @@ static ConvolveInfo *DestroyConvolveInfo(ConvolveInfo *convolve_info)
     status=clReleaseCommandQueue(convolve_info->command_queue);
   if (convolve_info->context != (cl_context) NULL)
     status=clReleaseContext(convolve_info->context);
+  (void) status;
   convolve_info=(ConvolveInfo *) RelinquishMagickMemory(convolve_info);
   return(convolve_info);
 }
 
 static MagickBooleanType EnqueueConvolveKernel(ConvolveInfo *convolve_info,
-  const Image *image,const void *pixels,double *filter,
-  const unsigned long width,const unsigned long height,void *convolve_pixels)
+  const Image *image,const void *pixels,float *filter,const size_t width,
+  const size_t height,void *convolve_pixels)
 {
   cl_int
     status;
@@ -434,7 +441,7 @@ static MagickBooleanType EnqueueConvolveKernel(ConvolveInfo *convolve_info,
     NULL);
   length=width*height;
   status=clEnqueueWriteBuffer(convolve_info->command_queue,
-    convolve_info->filter,CL_TRUE,0,length*sizeof(cl_double),filter,0,NULL,
+    convolve_info->filter,CL_TRUE,0,length*sizeof(cl_float),filter,0,NULL,
     NULL);
   if (status != CL_SUCCESS)
     return(MagickFalse);
@@ -462,9 +469,18 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
   char
     options[MaxTextExtent];
 
+  cl_context_properties
+    context_properties[3];
+
   cl_int
     status;
 
+  cl_platform_id
+    platforms[1];
+
+  cl_uint
+    number_platforms;
+
   ConvolveInfo
     *convolve_info;
 
@@ -475,7 +491,7 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
   /*
     Create OpenCL info.
   */
-  convolve_info=(ConvolveInfo *) AcquireAlignedMemory(1,sizeof(*convolve_info));
+  convolve_info=(ConvolveInfo *) AcquireMagickMemory(sizeof(*convolve_info));
   if (convolve_info == (ConvolveInfo *) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),
@@ -486,19 +502,32 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
   /*
     Create OpenCL context.
   */
-  convolve_info->context=clCreateContextFromType((cl_context_properties *)
-    NULL,CL_DEVICE_TYPE_GPU,ConvolveNotify,exception,&status);
+  status=clGetPlatformIDs(0,(cl_platform_id *) NULL,&number_platforms);
+  if ((status == CL_SUCCESS) && (number_platforms > 0))
+    status=clGetPlatformIDs(1,platforms,NULL);
+  if (status != CL_SUCCESS)
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),DelegateWarning,
+        "failed to create OpenCL context","`%s' (%d)",image->filename,status);
+      convolve_info=DestroyConvolveInfo(convolve_info);
+      return((ConvolveInfo *) NULL);
+    }
+  context_properties[0]=CL_CONTEXT_PLATFORM;
+  context_properties[1]=(cl_context_properties) platforms[0];
+  context_properties[2]=0;
+  convolve_info->context=clCreateContextFromType(context_properties,
+    (cl_device_type) CL_DEVICE_TYPE_GPU,ConvolveNotify,exception,&status);
   if ((convolve_info->context == (cl_context) NULL) || (status != CL_SUCCESS))
-    convolve_info->context=clCreateContextFromType((cl_context_properties *)
-      NULL,CL_DEVICE_TYPE_CPU,ConvolveNotify,exception,&status);
+    convolve_info->context=clCreateContextFromType(context_properties,
+      (cl_device_type) CL_DEVICE_TYPE_CPU,ConvolveNotify,exception,&status);
   if ((convolve_info->context == (cl_context) NULL) || (status != CL_SUCCESS))
-    convolve_info->context=clCreateContextFromType((cl_context_properties *)
-      NULL,CL_DEVICE_TYPE_DEFAULT,ConvolveNotify,exception,&status);
+    convolve_info->context=clCreateContextFromType(context_properties,
+      (cl_device_type) CL_DEVICE_TYPE_DEFAULT,ConvolveNotify,exception,&status);
   if ((convolve_info->context == (cl_context) NULL) || (status != CL_SUCCESS))
     {
       (void) ThrowMagickException(exception,GetMagickModule(),DelegateWarning,
         "failed to create OpenCL context","`%s' (%d)",image->filename,status);
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
   /*
@@ -508,7 +537,7 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
     &length);
   if ((status != CL_SUCCESS) || (length == 0))
     {
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
   convolve_info->devices=(cl_device_id *) AcquireMagickMemory(length);
@@ -516,16 +545,49 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),
         ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
   status=clGetContextInfo(convolve_info->context,CL_CONTEXT_DEVICES,length,
     convolve_info->devices,NULL);
   if (status != CL_SUCCESS)
     {
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
+  if (image->debug != MagickFalse)
+    {
+      char
+        attribute[MaxTextExtent];
+
+      size_t
+        length;
+
+      clGetDeviceInfo(convolve_info->devices[0],CL_DEVICE_NAME,
+        sizeof(attribute),attribute,&length);
+      (void) LogMagickEvent(AccelerateEvent,GetMagickModule(),"Name: %s",
+        attribute);
+      clGetDeviceInfo(convolve_info->devices[0],CL_DEVICE_VENDOR,
+        sizeof(attribute),attribute,&length);
+      (void) LogMagickEvent(AccelerateEvent,GetMagickModule(),"Vendor: %s",
+        attribute);
+      clGetDeviceInfo(convolve_info->devices[0],CL_DEVICE_VERSION,
+        sizeof(attribute),attribute,&length);
+      (void) LogMagickEvent(AccelerateEvent,GetMagickModule(),
+        "Driver Version: %s",attribute);
+      clGetDeviceInfo(convolve_info->devices[0],CL_DEVICE_PROFILE,
+        sizeof(attribute),attribute,&length);
+      (void) LogMagickEvent(AccelerateEvent,GetMagickModule(),"Profile: %s",
+        attribute);
+      clGetDeviceInfo(convolve_info->devices[0],CL_DRIVER_VERSION,
+        sizeof(attribute),attribute,&length);
+      (void) LogMagickEvent(AccelerateEvent,GetMagickModule(),"Driver: %s",
+        attribute);
+      clGetDeviceInfo(convolve_info->devices[0],CL_DEVICE_EXTENSIONS,
+        sizeof(attribute),attribute,&length);
+      (void) LogMagickEvent(AccelerateEvent,GetMagickModule(),"Extensions: %s",
+        attribute);
+    }
   /*
     Create OpenCL command queue.
   */
@@ -534,7 +596,7 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
   if ((convolve_info->command_queue == (cl_command_queue) NULL) ||
       (status != CL_SUCCESS))
     {
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
   /*
@@ -544,10 +606,10 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
     &source,lengths,&status);
   if ((convolve_info->program == (cl_program) NULL) || (status != CL_SUCCESS))
     {
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
-  (void) FormatMagickString(options,MaxTextExtent,CLOptions,(double)
+  (void) FormatLocaleString(options,MaxTextExtent,CLOptions,(float)
     QuantumRange,MagickEpsilon);
   status=clBuildProgram(convolve_info->program,1,convolve_info->devices,options,
     NULL,NULL);
@@ -561,7 +623,7 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
       log=(char *) AcquireMagickMemory(length);
       if (log == (char *) NULL)
         {
-          DestroyConvolveInfo(convolve_info);
+          convolve_info=DestroyConvolveInfo(convolve_info);
           return((ConvolveInfo *) NULL);
         }
       status=clGetProgramBuildInfo(convolve_info->program,
@@ -569,7 +631,7 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
       (void) ThrowMagickException(exception,GetMagickModule(),DelegateWarning,
         "failed to build OpenCL program","`%s' (%s)",image->filename,log);
       log=DestroyString(log);
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
   /*
@@ -578,7 +640,7 @@ static ConvolveInfo *GetConvolveInfo(const Image *image,const char *name,
   convolve_info->kernel=clCreateKernel(convolve_info->program,name,&status);
   if ((convolve_info->kernel == (cl_kernel) NULL) || (status != CL_SUCCESS))
     {
-      DestroyConvolveInfo(convolve_info);
+      convolve_info=DestroyConvolveInfo(convolve_info);
       return((ConvolveInfo *) NULL);
     }
   return(convolve_info);
@@ -601,6 +663,7 @@ MagickExport MagickBooleanType AccelerateConvolveImage(const Image *image,
   assert(exception->signature == MagickSignature);
   if ((image->storage_class != DirectClass) || 
       (image->colorspace == CMYKColorspace))
+    return(MagickFalse);
   if ((GetImageVirtualPixelMethod(image) != UndefinedVirtualPixelMethod) &&
       (GetImageVirtualPixelMethod(image) != EdgeVirtualPixelMethod))
     return(MagickFalse);
@@ -611,6 +674,9 @@ MagickExport MagickBooleanType AccelerateConvolveImage(const Image *image,
     const void
       *pixels;
 
+    float
+      *filter;
+
     ConvolveInfo
       *convolve_info;
 
@@ -620,6 +686,9 @@ MagickExport MagickBooleanType AccelerateConvolveImage(const Image *image,
     MagickSizeType
       length;
 
+    register ssize_t
+      i;
+
     void
       *convolve_pixels;
 
@@ -629,29 +698,43 @@ MagickExport MagickBooleanType AccelerateConvolveImage(const Image *image,
     pixels=AcquirePixelCachePixels(image,&length,exception);
     if (pixels == (const void *) NULL)
       {
+        convolve_info=DestroyConvolveInfo(convolve_info);
         (void) ThrowMagickException(exception,GetMagickModule(),CacheError,
           "UnableToReadPixelCache","`%s'",image->filename);
-        convolve_info=DestroyConvolveInfo(convolve_info);
         return(MagickFalse);
       }
     convolve_pixels=GetPixelCachePixels(convolve_image,&length,exception);
     if (convolve_pixels == (void *) NULL)
       {
+        convolve_info=DestroyConvolveInfo(convolve_info);
         (void) ThrowMagickException(exception,GetMagickModule(),CacheError,
           "UnableToReadPixelCache","`%s'",image->filename);
-        convolve_info=DestroyConvolveInfo(convolve_info);
         return(MagickFalse);
       }
-    status=BindConvolveParameters(convolve_info,image,pixels,kernel->values,
+    filter=(float *) AcquireQuantumMemory(kernel->width,kernel->height*
+      sizeof(*filter));
+    if (filter == (float *) NULL)
+      {
+        DestroyConvolveBuffers(convolve_info);
+        convolve_info=DestroyConvolveInfo(convolve_info);
+        (void) ThrowMagickException(exception,GetMagickModule(),
+          ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
+        return(MagickFalse);
+      }
+    for (i=0; i < (ssize_t) (kernel->width*kernel->height); i++)
+      filter[i]=(float) kernel->values[i];
+    status=BindConvolveParameters(convolve_info,image,pixels,filter,
       kernel->width,kernel->height,convolve_pixels);
     if (status == MagickFalse)
       {
+        filter=(float *) RelinquishMagickMemory(filter);
         DestroyConvolveBuffers(convolve_info);
         convolve_info=DestroyConvolveInfo(convolve_info);
         return(MagickFalse);
       }
-    status=EnqueueConvolveKernel(convolve_info,image,pixels,kernel->values,
+    status=EnqueueConvolveKernel(convolve_info,image,pixels,filter,
       kernel->width,kernel->height,convolve_pixels);
+    filter=(float *) RelinquishMagickMemory(filter);
     if (status == MagickFalse)
       {
         DestroyConvolveBuffers(convolve_info);
