@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -95,11 +95,11 @@ static const CoderMapInfo
     { "BMP3", "BMP" },
     { "B", "RAW" },
     { "BRF", "BRAILLE" },
-    { "BGR", "RGB" },
-    { "BRG", "RGB" },
+    { "BGRA", "BGR" },
     { "CMYKA", "CMYK" },
     { "C", "RAW" },
     { "CAL", "CALS" },
+    { "CANVAS", "XC" },
     { "CR2", "DNG" },
     { "CRW", "DNG" },
     { "CUR", "ICON" },
@@ -124,8 +124,6 @@ static const CoderMapInfo
     { "G3", "FAX" },
     { "GIF87", "GIF" },
     { "G", "RAW" },
-    { "GBR", "RGB" },
-    { "GRB", "RGB" },
     { "GRANITE", "MAGICK" },
     { "GROUP4", "TIFF" },
     { "K25", "DNG" },
@@ -146,6 +144,7 @@ static const CoderMapInfo
     { "JNG", "PNG" },
     { "JPC", "JP2" },
     { "J2C", "JP2" },
+    { "J2K", "JP2" },
     { "JPG", "JPEG" },
     { "JPX", "JP2" },
     { "K", "RAW" },
@@ -154,14 +153,16 @@ static const CoderMapInfo
     { "M4V", "MPEG" },
     { "M", "RAW" },
     { "MNG", "PNG" },
-    { "MOV", "PNG" },
+    { "MOV", "MPEG" },
     { "MP4", "MPEG" },
     { "MPG", "MPEG" },
     { "MPRI", "MPR" },
+    { "MEF", "DNG" },
     { "MRW", "DNG" },
     { "MSVG", "SVG" },
     { "NEF", "DNG" },
     { "NETSCAPE", "MAGICK" },
+    { "NRW", "DNG" },
     { "O", "RAW" },
     { "ORF", "DNG" },
     { "OTF", "TTF" },
@@ -190,7 +191,6 @@ static const CoderMapInfo
     { "RADIAL-GRADIENT", "GRADIENT" },
     { "RAF", "DNG" },
     { "RAS", "SUN" },
-    { "RBG", "RGB" },
     { "RGBA", "RGB" },
     { "RGBO", "RGB" },
     { "R", "RAW" },
@@ -206,6 +206,7 @@ static const CoderMapInfo
     { "UBRL", "BRAILLE" },
     { "VDA", "TGA" },
     { "VST", "TGA" },
+    { "WIZARD", "MAGICK" },
     { "WMV", "MPEG" },
     { "WMFWIN32", "EMF" },
     { "WMZ", "WMF" },
@@ -213,10 +214,8 @@ static const CoderMapInfo
     { "XMP", "META" },
     { "XTRNARRAY", "XTRN" },
     { "XTRNBLOB", "XTRN" },
-    { "XTRNBSTR", "XTRN" },
     { "XTRNFILE", "XTRN" },
     { "XTRNIMAGE", "XTRN" },
-    { "XTRNSTREAM", "XTRN" },
     { "XV", "VIFF" },
     { "Y", "RAW" },
     { "YCbCrA", "YCbCr" }
@@ -581,15 +580,15 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
   const CoderInfo
     **coder_info;
 
-  ssize_t
-    j;
-
   register ssize_t
     i;
 
   size_t
     number_coders;
 
+  ssize_t
+    j;
+
   if (file == (const FILE *) NULL)
     file=stdout;
   coder_info=GetCoderInfoList("*",&number_coders,exception);
@@ -604,18 +603,19 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
         (LocaleCompare(path,coder_info[i]->path) != 0))
       {
         if (coder_info[i]->path != (char *) NULL)
-          (void) fprintf(file,"\nPath: %s\n\n",coder_info[i]->path);
-        (void) fprintf(file,"Magick      Coder\n");
-        (void) fprintf(file,"-------------------------------------------------"
+          (void) FormatLocaleFile(file,"\nPath: %s\n\n",coder_info[i]->path);
+        (void) FormatLocaleFile(file,"Magick      Coder\n");
+        (void) FormatLocaleFile(file,
+          "-------------------------------------------------"
           "------------------------------\n");
       }
     path=coder_info[i]->path;
-    (void) fprintf(file,"%s",coder_info[i]->magick);
+    (void) FormatLocaleFile(file,"%s",coder_info[i]->magick);
     for (j=(ssize_t) strlen(coder_info[i]->magick); j <= 11; j++)
-      (void) fprintf(file," ");
+      (void) FormatLocaleFile(file," ");
     if (coder_info[i]->name != (char *) NULL)
-      (void) fprintf(file,"%s",coder_info[i]->name);
-    (void) fprintf(file,"\n");
+      (void) FormatLocaleFile(file,"%s",coder_info[i]->name);
+    (void) FormatLocaleFile(file,"\n");
   }
   coder_info=(const CoderInfo **) RelinquishMagickMemory((void *) coder_info);
   (void) fflush(file);
@@ -782,7 +782,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
         /*
           Coder element.
         */
-        coder_info=(CoderInfo *) AcquireAlignedMemory(1,sizeof(*coder_info));
+        coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
         if (coder_info == (CoderInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
@@ -913,7 +913,7 @@ static MagickBooleanType LoadCoderLists(const char *filename,
       *p;
 
     p=CoderMap+i;
-    coder_info=(CoderInfo *) AcquireAlignedMemory(1,sizeof(*coder_info));
+    coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
     if (coder_info == (CoderInfo *) NULL)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),
