@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ extern "C" {
 #include <magick/exception-private.h>
 
 static inline IndexPacket ConstrainColormapIndex(Image *image,
-  const unsigned long index)
+  const size_t index)
 {
   if (index < image->colors)
     return((IndexPacket) index);
