@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -31,8 +31,8 @@ extern MagickExport FxInfo
 
 extern MagickExport MagickBooleanType
   FxEvaluateExpression(FxInfo *,MagickRealType *,ExceptionInfo *),
-  FxEvaluateChannelExpression(FxInfo *,const ChannelType,const long,const long,
-    MagickRealType *,ExceptionInfo *);
+  FxEvaluateChannelExpression(FxInfo *,const ChannelType,const ssize_t,
+   const ssize_t,MagickRealType *,ExceptionInfo *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
