@@ -17,7 +17,7 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -67,6 +67,16 @@
 %  whether the x or y values are negative.  In addition, there are flags to
 %  report any meta characters (%, !, <, or >).
 %
+%  The value must form a proper geometry style specification of WxH+X+Y
+%  of integers only, and values can not be separated by comma, colon, or
+%  slash charcaters.  See ParseGeometry() below.
+%
+%  Offsets may be prefixed by multiple signs to make offset string
+%  substitutions easier to handle from shell scripts.
+%  For example: "-10-10", "-+10-+10", or "+-10+-10" will generate negtive
+%  offsets, while "+10+10", "++10++10", or "--10--10" will generate positive
+%  offsets.
+%
 %  The format of the GetGeometry method is:
 %
 %      MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_t *y,
@@ -82,8 +92,8 @@
 %      specification.
 %
 */
-MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_t *y,
-  size_t *width,size_t *height)
+MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
+  ssize_t *y,size_t *width,size_t *height)
 {
   char
     *p,
@@ -93,6 +103,9 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
   double
     value;
 
+  int
+    c;
+
   MagickStatusType
     flags;
 
@@ -102,7 +115,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
   flags=NoValue;
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(flags);
-  if (strlen(geometry) >= MaxTextExtent)
+  if (strlen(geometry) >= (MaxTextExtent-1))
     return(flags);
   (void) CopyMagickString(pedantic_geometry,geometry,MaxTextExtent);
   for (p=pedantic_geometry; *p != '\0'; )
@@ -112,7 +125,8 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
         (void) CopyMagickString(p,p+1,MaxTextExtent);
         continue;
       }
-    switch ((int) *p)
+    c=(int)*p;
+    switch (c)
     {
       case '%':
       {
@@ -170,9 +184,9 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
       case '7':
       case '8':
       case '9':
-      case -41:
       case 'x':
       case 'X':
+      case 215:
       {
         p++;
         break;
@@ -188,10 +202,12 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
   if (*p == '\0')
     return(flags);
   q=p;
-  value=strtod(p,&q);
+  value=StringToDouble(p,&q);
+  (void) value;
   if (LocaleNCompare(p,"0x",2) == 0)
     value=(double) strtol(p,&q,10);
-  if ((((int) *q) == -41) || (*q == 'x') || (*q == 'X') || (*q == '\0'))
+  c=(int) ((unsigned char) *q);
+  if ((c == 215) || (*q == 'x') || (*q == 'X') || (*q == '\0'))
     {
       /*
         Parse width.
@@ -200,11 +216,12 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
       if (LocaleNCompare(p,"0x",2) == 0)
         *width=(size_t) strtol(p,&p,10);
       else
-        *width=(size_t) floor(strtod(p,&p)+0.5);
+        *width=(size_t) floor(StringToDouble(p,&p)+0.5);
       if (p != q)
         flags|=WidthValue;
     }
-  if ((((int) *p) == -41) || (*p == 'x') || (*p == 'X'))
+  c=(int) ((unsigned char) *p);
+  if ((c == 215) || (*p == 'x') || (*p == 'X'))
     {
       p++;
       if ((*p != '+') && (*p != '-'))
@@ -213,7 +230,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
             Parse height.
           */
           q=p;
-          *height=(size_t) floor(strtod(p,&p)+0.5);
+          *height=(size_t) floor(StringToDouble(p,&p)+0.5);
           if (p != q)
             flags|=HeightValue;
         }
@@ -223,25 +240,52 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
       /*
         Parse x value.
       */
-      if (*p == '-')
-        flags|=XNegative;
-      q=p;
-      *x=(ssize_t) ceil(strtod(p,&p)-0.5);
-      if (p != q)
-        flags|=XValue;
-      if ((*p == '+') || (*p == '-'))
+      while ((*p == '+') || (*p == '-'))
         {
-          /*
-            Parse y value.
-          */
           if (*p == '-')
-            flags|=YNegative;
-          q=p;
-          *y=(ssize_t) ceil(strtod(p,&p)-0.5);
-          if (p != q)
-            flags|=YValue;
+            flags^=XNegative;  /* negate sign */
+          p++;
+        }
+      q=p;
+      *x=(ssize_t) ceil(StringToDouble(p,&p)-0.5);
+      if (p != q)
+        {
+          flags|=XValue;
+          if ( (flags&XNegative) != 0 )
+            *x = -*x;
         }
     }
+  if ((*p == '+') || (*p == '-'))
+    {
+      /*
+        Parse y value.
+      */
+      while ((*p == '+') || (*p == '-'))
+        {
+          if (*p == '-')
+            flags^=YNegative;  /* negate sign */
+          p++;
+        }
+      q=p;
+      *y=(ssize_t) ceil(StringToDouble(p,&p)-0.5);
+      if (p != q)
+        {
+          flags|=YValue;
+          if ( (flags&YNegative) != 0 )
+            *y = -*y;
+        }
+    }
+#if 0
+  /* Debugging Geometry */
+  fprintf(stderr,"GetGeometry...\n");
+  fprintf(stderr,"Input: %s\n",geometry);
+  fprintf(stderr,"Flags: %c %c %s %s\n",
+       (flags&WidthValue)?'W':' ',(flags&HeightValue)?'H':' ',
+       (flags&XValue)?((flags&XNegative)?"-X":"+X"):"  ",
+       (flags&YValue)?((flags&YNegative)?"-Y":"+Y"):"  ");
+  fprintf(stderr,"Geometry: %ldx%ld%+ld%+ld\n",
+       (long)*width,(long)*height,(long)*x,(long)*y);
+#endif
   return(flags);
 }
 
@@ -265,9 +309,9 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,ssize_
 %
 %  A description of each parameter follows.
 %
-%   o  page_geometry:  Specifies a pointer to an array of characters.
-%      The string is either a Postscript page name (e.g. A4) or a postscript
-%      page geometry (e.g. 612x792+36+36).
+%   o  page_geometry:  Specifies a pointer to an array of characters.  The
+%      string is either a Postscript page name (e.g. A4) or a postscript page
+%      geometry (e.g. 612x792+36+36).
 %
 */
 MagickExport char *GetPageGeometry(const char *page_geometry)
@@ -393,8 +437,7 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %                                                                             %
 %   G r a v i t y A d j u s t G e o m e t r y                                 %
 %                                                                             %
-%                                                                             %
-%                                                                             %
+%                                                                             % %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GravityAdjustGeometry() adjusts the offset of a region with regard to the
@@ -405,9 +448,8 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %
 %  The format of the GravityAdjustGeometry method is:
 %
-%      void GravityAdjustGeometry(const size_t width,
-%        const size_t height,const GravityType gravity,
-%        RectangleInfo *region);
+%      void GravityAdjustGeometry(const size_t width, const size_t height,
+%        const GravityType gravity,RectangleInfo *region);
 %
 %  A description of each parameter follows:
 %
@@ -510,8 +552,6 @@ MagickExport MagickBooleanType IsGeometry(const char *geometry)
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(geometry,&geometry_info);
-  if (flags == NoValue)
-    flags=ParseGeometry(geometry+1,&geometry_info);  /* i.e. +-4+-4 */
   return(flags != NoValue ? MagickTrue : MagickFalse);
 }
 
@@ -554,7 +594,8 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   p=(char *) geometry;
-  value=strtod(geometry,&p);
+  value=StringToDouble(geometry,&p);
+  (void) value;
   if (p == geometry)
     return(MagickFalse);
   if (strspn(geometry,"0123456789-, ") != strlen(geometry))
@@ -575,7 +616,11 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ParseAbsoluteGeometry() returns a region as defined by the geometry string.
+%  ParseAbsoluteGeometry() returns a region as defined by the geometry string,
+%  without any modification by percentages or gravity.
+%
+%  It currently just a wrapper around GetGeometry(), but may be expanded in
+%  the future to handle other positioning information.
 %
 %  The format of the ParseAbsoluteGeometry method is:
 %
@@ -584,7 +629,7 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry (e.g. 100x100+10+10).
+%    o geometry:  The geometry string (e.g. "100x100+10+10").
 %
 %    o region_info: the region as defined by the geometry string.
 %
@@ -611,8 +656,10 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ParseAffineGeometry() returns an affine matrix as defined by the geometry
-%  string.
+%  ParseAffineGeometry() returns an affine matrix as defined by a string of 4
+%  to 6 comma/space separated floating point values.
+%
+%  The affine matrix determinant is checked for validity of the values.
 %
 %  The format of the ParseAffineGeometry method is:
 %
@@ -621,7 +668,7 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry (e.g. 1.0,0.0,0.0,1.0,3.2,1.2).
+%    o geometry:  The geometry string (e.g. "1.0,0.0,0.0,1.0,3.2,1.2").
 %
 %    o affine_matrix: the affine matrix as defined by the geometry string.
 %
@@ -656,19 +703,45 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
       GetMagickToken(p,&p,token);
     switch (i)
     {
-      case 0: affine_matrix->sx=StringToDouble(token); break;
-      case 1: affine_matrix->rx=StringToDouble(token); break;
-      case 2: affine_matrix->ry=StringToDouble(token); break;
-      case 3: affine_matrix->sy=StringToDouble(token); break;
-      case 4: affine_matrix->tx=StringToDouble(token); flags|=XValue; break;
-      case 5: affine_matrix->ty=StringToDouble(token); flags|=YValue; break;
+      case 0:
+      {
+        affine_matrix->sx=StringToDouble(token,(char **) NULL);
+        break;
+      }
+      case 1:
+      {
+        affine_matrix->rx=StringToDouble(token,(char **) NULL);
+        break;
+      }
+      case 2:
+      {
+        affine_matrix->ry=StringToDouble(token,(char **) NULL);
+        break;
+      }
+      case 3:
+      {
+        affine_matrix->sy=StringToDouble(token,(char **) NULL);
+        break;
+      }
+      case 4:
+      {
+        affine_matrix->tx=StringToDouble(token,(char **) NULL);
+        flags|=XValue;
+        break;
+      }
+      case 5:
+      {
+        affine_matrix->ty=StringToDouble(token,(char **) NULL);
+        flags|=YValue;
+        break;
+      }
     }
   }
-  determinant=(affine_matrix->sx*affine_matrix->sy-affine_matrix->rx*
-    affine_matrix->ry);
+  determinant=(affine_matrix->sx*affine_matrix->sy
+                 - affine_matrix->rx*affine_matrix->ry);
   if (fabs(determinant) < MagickEpsilon)
     (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
-      "InvalidGeometry","`%s'",geometry);
+      "InvalidArgument","'%s' : 'Indeterminate Matrix'",geometry);
   return(flags);
 }
 
@@ -686,8 +759,18 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
 %  ParseGeometry() parses a geometry specification and returns the sigma,
 %  rho, xi, and psi values.  It also returns flags that indicates which
 %  of the four values (sigma, rho, xi, psi) were located in the string, and
-%  whether the xi or pi values are negative.  In addition, there are flags to
-%  report any meta characters (%, !, <, or >).
+%  whether the xi or pi values are negative.
+%
+%  In addition, it reports if there are any of meta characters (%, !, <, >, @,
+%  and ^) flags present. It does not report the location of the percentage
+%  relative to the values.
+%
+%  Values may also be seperated by commas, colons, or slashes, and offsets.
+%  Offsets may be prefixed by multiple signs to make offset string
+%  substitutions easier to handle from shell scripts.
+%  For example: "-10-10", "-+10-+10", or "+-10+-10" will generate negtive
+%  offsets, while "+10+10", "++10++10", or "--10--10" will generate positive
+%  offsets.
 %
 %  The format of the ParseGeometry method is:
 %
@@ -696,7 +779,7 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry.
+%    o geometry:  The geometry string (e.g. "100x100+10+10").
 %
 %    o geometry_info:  returns the parsed width/height/x/y in this structure.
 %
@@ -712,6 +795,9 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   double
     value;
 
+  int
+    c;
+
   MagickStatusType
     flags;
 
@@ -722,7 +808,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   flags=NoValue;
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(flags);
-  if (strlen(geometry) >= MaxTextExtent)
+  if (strlen(geometry) >= (MaxTextExtent-1))
     return(flags);
   (void) CopyMagickString(pedantic_geometry,geometry,MaxTextExtent);
   for (p=pedantic_geometry; *p != '\0'; )
@@ -732,7 +818,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         (void) CopyMagickString(p,p+1,MaxTextExtent);
         continue;
       }
-    switch ((int) *p)
+    c=(int) ((unsigned char) *p);
+    switch (c)
     {
       case '%':
       {
@@ -789,11 +876,11 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       case '7':
       case '8':
       case '9':
-      case -41:
       case 'x':
       case 'X':
       case '/':
       case ':':
+      case 215:
       {
         p++;
         break;
@@ -815,10 +902,11 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   if (*p == '\0')
     return(flags);
   q=p;
-  value=strtod(p,&q);
+  value=StringToDouble(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
     value=(double) strtol(p,&q,10);
-  if ((((int) *q) == -41) || (*q == 'x') || (*q == 'X') || (*q == ',') ||
+  c=(int) ((unsigned char) *q);
+  if ((c == 215) || (*q == 'x') || (*q == 'X') || (*q == ',') ||
       (*q == '/') || (*q == ':') || (*q =='\0'))
     {
       /*
@@ -828,7 +916,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       if (LocaleNCompare(p,"0x",2) == 0)
         value=(double) strtol(p,&p,10);
       else
-        value=strtod(p,&p);
+        value=StringToDouble(p,&p);
       if (p != q)
         {
           flags|=RhoValue;
@@ -836,8 +924,9 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
     }
   q=p;
-  if ((((int) *p) == -41) || (*p == 'x') || (*p == 'X') || (*p == ',') ||
-      (*p == '/') || (*p == ':'))
+  c=(int) ((unsigned char) *p);
+  if ((c == 215) || (*p == 'x') || (*p == 'X') || (*p == ',') || (*p == '/') ||
+      (*p == ':'))
     {
       /*
         Parse sigma.
@@ -845,11 +934,12 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       p++;
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
-      if (((((int) *q) != -41) && (*q != 'x') && (*q != 'X')) ||
-          ((*p != '+') && (*p != '-')))
+      c=(int) ((unsigned char) *q);
+      if (((c != 215) && (*q != 'x') && (*q != 'X')) || ((*p != '+') &&
+          (*p != '-')))
         {
           q=p;
-          value=strtod(p,&p);
+          value=StringToDouble(p,&p);
           if (p != q)
             {
               flags|=SigmaValue;
@@ -864,15 +954,21 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       /*
         Parse xi value.
       */
-      if ((*p == ',') || (*p == '/') || (*p == ':'))
+      if ((*p == ',') || (*p == '/') || (*p == ':') )
         p++;
+      while ((*p == '+') || (*p == '-'))
+        {
+          if (*p == '-')
+            flags^=XiNegative;  /* negate sign */
+          p++;
+        }
       q=p;
-      value=strtod(p,&p);
+      value=StringToDouble(p,&p);
       if (p != q)
         {
           flags|=XiValue;
-          if (*q == '-')
-            flags|=XiNegative;
+          if ( (flags&XiNegative) != 0 )
+            value = -value;
           geometry_info->xi=value;
         }
       while (isspace((int) ((unsigned char) *p)) != 0)
@@ -885,15 +981,20 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           */
           if ((*p == ',') || (*p == '/') || (*p == ':'))
             p++;
-          q=p;
-          value=strtod(p,&p);
-          if (p != q)
+          while ((*p == '+') || (*p == '-'))
             {
-              flags|=PsiValue;
-              if (*q == '-')
-                flags|=PsiNegative;
-              geometry_info->psi=value;
+              if (*p == '-')
+                flags^=PsiNegative;  /* negate sign */
+              p++;
             }
+          q=p;
+          value=StringToDouble(p,&p);
+          if (p != q) {
+            flags|=PsiValue;
+            if ( (flags&PsiNegative) != 0 )
+              value = -value;
+            geometry_info->psi=value;
+          }
         }
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
@@ -905,15 +1006,20 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           */
           if ((*p == ',') || (*p == '/') || (*p == ':'))
             p++;
-          q=p;
-          value=strtod(p,&p);
-          if (p != q)
+          while ((*p == '+') || (*p == '-'))
             {
-              flags|=ChiValue;
-              if (*q == '-')
-                flags|=ChiNegative;
-              geometry_info->chi=value;
+              if (*p == '-')
+                flags^=ChiNegative;  /* negate sign */
+              p++;
             }
+          q=p;
+          value=StringToDouble(p,&p);
+          if (p != q) {
+            flags|=ChiValue;
+            if ( (flags&ChiNegative) != 0 )
+              value = -value;
+            geometry_info->chi=value;
+          }
         }
     }
   if (strchr(pedantic_geometry,':') != (char *) NULL)
@@ -937,6 +1043,18 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       flags|=SigmaValue;
       flags&=(~XiValue);
     }
+#if 0
+  /* Debugging Geometry */
+  fprintf(stderr,"ParseGeometry...\n");
+  fprintf(stderr,"Flags: %c %c %s %s %s\n",
+       (flags&RhoValue)?'W':' ',(flags&SigmaValue)?'H':' ',
+       (flags&XiValue)?((flags&XiNegative)?"-X":"+X"):"  ",
+       (flags&PsiValue)?((flags&PsiNegative)?"-Y":"+Y"):"  ",
+       (flags&ChiValue)?((flags&ChiNegative)?"-Z":"+Z"):"  ");
+  fprintf(stderr,"Geometry: %lg,%lg,%lg,%lg,%lg\n",
+       geometry_info->rho,geometry_info->sigma,
+       geometry_info->xi,geometry_info->psi,geometry_info->chi);
+#endif
   return(flags);
 }
 
@@ -952,7 +1070,15 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ParseGravityGeometry() returns a region as defined by the geometry string
-%  with respect to the image dimensions and its gravity.
+%  with respect to the given image page (canvas) dimensions and the images
+%  gravity setting.
+%
+%  This is typically used for specifing a area within a given image for
+%  cropping images to a smaller size, chopping out rows and or columns, or
+%  resizing and positioning overlay images.
+%
+%  Percentages are relative to image size and not page size, and are set to
+%  nearest integer (pixel) size.
 %
 %  The format of the ParseGravityGeometry method is:
 %
@@ -961,10 +1087,10 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry (e.g. 100x100+10+10).
+%    o geometry:  The geometry string (e.g. "100x100+10+10").
 %
-%    o region_info: the region as defined by the geometry string with
-%      respect to the image dimensions and its gravity.
+%    o region_info: the region as defined by the geometry string with respect
+%      to the image dimensions and its gravity.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -1003,7 +1129,7 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
         scale;
 
       /*
-        Geometry is a percentage of the image size.
+        Geometry is a percentage of the image size, not canvas size
       */
       if (image->gravity != UndefinedGravity)
         flags|=XValue | YValue;
@@ -1014,10 +1140,8 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
       scale.y=geometry_info.sigma;
       if ((status & SigmaValue) == 0)
         scale.y=scale.x;
-      region_info->width=(size_t) floor((scale.x*image->columns/100.0)+
-        0.5);
-      region_info->height=(size_t) floor((scale.y*image->rows/100.0)+
-        0.5);
+      region_info->width=(size_t) floor((scale.x*image->columns/100.0)+0.5);
+      region_info->height=(size_t) floor((scale.y*image->rows/100.0)+0.5);
     }
   /*
     Adjust offset according to gravity setting.
@@ -1046,22 +1170,34 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ParseMetaGeometry() is similar to GetGeometry() except the returned
-%  geometry is modified as determined by the meta characters:  %, !, <, >,
-%  and ~.
+%  geometry is modified as determined by the meta characters:  %, !, <, >, @,
+%  and ^ in relation to image resizing.
+%
+%  Final image dimensions are adjusted so as to preserve the aspect ratio as
+%  much as possible, while generating a integer (pixel) size, and fitting the
+%  image within the specified geometry width and height.
+%
+%  Flags are interpreted...
+%     %   geometry size is given percentage of original width and height given
+%     !   do not try to preserve aspect ratio
+%     <   only enlarge images smaller that geometry
+%     >   only shrink images larger than geometry
+%     @   Fit image to contain at most this many pixels
+%     ^   Contain the given geometry given, (minimal dimensions given)
 %
 %  The format of the ParseMetaGeometry method is:
 %
-%      MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,ssize_t *y,
-%        size_t *width,size_t *height)
+%      MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
+%        ssize_t *y, size_t *width,size_t *height)
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry.
+%    o geometry:  The geometry string (e.g. "100x100+10+10").
 %
-%    o x,y:  The x and y offset as determined by the geometry specification.
+%    o x,y:  The x and y offset, set according to the geometry specification.
 %
-%    o width,height:  The width and height as determined by the geometry
-%      specification.
+%    o width,height:  The width and height of original image, modified by
+%      the given geometry specification.
 %
 */
 
@@ -1130,10 +1266,17 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       former_width=(*width);
       former_height=(*height);
     }
-  if (((flags & AspectValue) == 0) && ((*width != former_width) ||
-      (*height != former_height)))
+  if (((flags & AspectValue) != 0) || ((*width == former_width) &&
+      (*height == former_height)))
     {
-      MagickRealType
+      if ((flags & RhoValue) == 0)
+        *width=former_width;
+      if ((flags & SigmaValue) == 0)
+        *height=former_height;
+    }
+  else
+    {
+      double
         scale_factor;
 
       /*
@@ -1144,45 +1287,33 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       else
         if (((flags & RhoValue) != 0) && (flags & SigmaValue) != 0)
           {
-            scale_factor=(MagickRealType) *width/(MagickRealType) former_width;
+            scale_factor=(double) *width/(double) former_width;
             if ((flags & MinimumValue) == 0)
               {
-                if (scale_factor > ((MagickRealType) *height/
-                    (MagickRealType) former_height))
-                  scale_factor=(MagickRealType) *height/(MagickRealType)
-                    former_height;
+                if (scale_factor > ((double) *height/(double) former_height))
+                  scale_factor=(double) *height/(double) former_height;
               }
             else
-              if (scale_factor < ((MagickRealType) *height/
-                  (MagickRealType) former_height))
-                scale_factor=(MagickRealType) *height/(MagickRealType)
-                  former_height;
+              if (scale_factor < ((double) *height/(double) former_height))
+                scale_factor=(double) *height/(double) former_height;
           }
         else
           if ((flags & RhoValue) != 0)
             {
-              scale_factor=(MagickRealType) *width/(MagickRealType)
-                former_width;
+              scale_factor=(double) *width/(double) former_width;
               if (((flags & MinimumValue) != 0) &&
-                  (scale_factor < ((MagickRealType) *width/
-                    (MagickRealType) former_height)))
-                scale_factor=(MagickRealType) *width/(MagickRealType)
-                  former_height;
+                  (scale_factor < ((double) *width/(double) former_height)))
+                scale_factor=(double) *width/(double) former_height;
             }
           else
             {
-              scale_factor=(MagickRealType) *height/(MagickRealType)
-                former_height;
+              scale_factor=(double) *height/(double) former_height;
               if (((flags & MinimumValue) != 0) &&
-                  (scale_factor < ((MagickRealType) *height/
-                    (MagickRealType) former_width)))
-                scale_factor=(MagickRealType) *height/(MagickRealType)
-                  former_width;
+                  (scale_factor < ((double) *height/(double) former_width)))
+                scale_factor=(double) *height/(double) former_width;
             }
-      *width=MagickMax((size_t) floor(scale_factor*former_width+0.5),
-        1UL);
-      *height=MagickMax((size_t) floor(scale_factor*former_height+0.5),
-        1UL);
+      *width=MagickMax((size_t) floor(scale_factor*former_width+0.5),1UL);
+      *height=MagickMax((size_t) floor(scale_factor*former_height+0.5),1UL);
     }
   if ((flags & GreaterValue) != 0)
     {
@@ -1200,7 +1331,7 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
     }
   if ((flags & AreaValue) != 0)
     {
-      MagickRealType
+      double
         area,
         distance;
 
@@ -1213,13 +1344,12 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho;
       distance=sqrt((double) former_width*former_height);
-      scale.x=(double) former_width/(double) (distance/sqrt((double) area));
-      scale.y=(double) former_height/(double) (distance/sqrt((double) area));
+      scale.x=(double) former_width/(distance/sqrt(area));
+      scale.y=(double) former_height/(distance/sqrt(area));
       if ((scale.x < (double) *width) || (scale.y < (double) *height))
         {
-          *width=(size_t) (former_width/(distance/sqrt((double) area)));
-          *height=(size_t) (former_height/(distance/
-            sqrt((double) area)));
+          *width=(size_t) (former_width/(distance/sqrt(area))+0.5);
+          *height=(size_t) (former_height/(distance/sqrt(area))+0.5);
         }
       former_width=(*width);
       former_height=(*height);
@@ -1239,7 +1369,10 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ParsePageGeometry() returns a region as defined by the geometry string with
-%  respect to the image dimensions.
+%  respect to the image page (canvas) dimensions.
+%
+%  WARNING: Percentage dimensions remain relative to the actual image
+%  dimensions, and not canvas dimensions.
 %
 %  The format of the ParsePageGeometry method is:
 %
@@ -1249,7 +1382,7 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry (e.g. 100x100+10+10).
+%    o geometry:  The geometry string (e.g. "100x100+10+10").
 %
 %    o region_info: the region as defined by the geometry string with
 %      respect to the image and its gravity.
@@ -1299,6 +1432,10 @@ MagickExport MagickStatusType ParsePageGeometry(const Image *image,
 %  ParseRegionGeometry() returns a region as defined by the geometry string
 %  with respect to the image dimensions and aspect ratio.
 %
+%  This is basically a wrapper around ParseMetaGeometry.  This is typically
+%  used to parse a geometry string to work out the final integer dimensions
+%  for image resizing.
+%
 %  The format of the ParseRegionGeometry method is:
 %
 %      MagickStatusType ParseRegionGeometry(const Image *image,
@@ -1307,7 +1444,7 @@ MagickExport MagickStatusType ParsePageGeometry(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o geometry:  The geometry (e.g. 100x100+10+10).
+%    o geometry:  The geometry string (e.g. "100x100+10+10").
 %
 %    o region_info: the region as defined by the geometry string.
 %
