@@ -17,7 +17,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,13 +54,10 @@
 #include "magick/monitor-private.h"
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
+#include "magick/resource_.h"
 #include "magick/string_.h"
 #include "magick/thread-private.h"
 
-static inline double MagickMax(const double x,const double y)
-{
-    return( x > y ? x : y);
-}
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -88,8 +85,8 @@ static inline double MagickMax(const double x,const double y)
 %
 %      MagickBooleanType FloodfillPaintImage(Image *image,
 %        const ChannelType channel,const DrawInfo *draw_info,
-%        const MagickPixelPacket target,const ssize_t x_offset,const ssize_t y_offset,
-%        const MagickBooleanType invert)
+%        const MagickPixelPacket target,const ssize_t x_offset,
+%        const ssize_t y_offset,const MagickBooleanType invert)
 %
 %  A description of each parameter follows:
 %
@@ -129,20 +126,16 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     } \
 }
 
+  CacheView
+    *floodplane_view,
+    *image_view;
+
   ExceptionInfo
     *exception;
 
   Image
     *floodplane_image;
 
-  ssize_t
-    offset,
-    start,
-    x,
-    x1,
-    x2,
-    y;
-
   MagickBooleanType
     skip;
 
@@ -159,6 +152,14 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   SegmentInfo
     *segment_stack;
 
+  ssize_t
+    offset,
+    start,
+    x,
+    x1,
+    x2,
+    y;
+
   /*
     Check boundary conditions.
   */
@@ -174,7 +175,10 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     return(MagickFalse);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  if (image->matte == MagickFalse)
+  if (IsGrayColorspace(image->colorspace) != MagickFalse)
+    (void) TransformImageColorspace(image,sRGBColorspace);
+  if ((image->matte == MagickFalse) &&
+      (draw_info->fill.opacity != OpaqueOpacity))
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   /*
     Set floodfill state.
@@ -203,6 +207,8 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   PushSegmentStack(y+1,x,x,-1);
   GetMagickPixelPacket(image,&fill);
   GetMagickPixelPacket(image,&pixel);
+  image_view=AcquireVirtualCacheView(image,exception);
+  floodplane_view=AcquireAuthenticCacheView(floodplane_image,exception);
   while (s > segment_stack)
   {
     register const IndexPacket
@@ -228,12 +234,12 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     /*
       Recolor neighboring pixels.
     */
-    p=GetVirtualPixels(image,0,y,(size_t) (x1+1),1,exception);
-    q=GetAuthenticPixels(floodplane_image,0,y,(size_t) (x1+1),1,
+    p=GetCacheViewVirtualPixels(image_view,0,y,(size_t) (x1+1),1,exception);
+    q=GetCacheViewAuthenticPixels(floodplane_view,0,y,(size_t) (x1+1),1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetVirtualIndexQueue(image);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
     p+=x1;
     q+=x1;
     for (x=x1; x >= 0; x--)
@@ -247,7 +253,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       p--;
       q--;
     }
-    if (SyncAuthenticPixels(floodplane_image,exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(floodplane_view,exception) == MagickFalse)
       break;
     skip=x >= x1 ? MagickTrue : MagickFalse;
     if (skip == MagickFalse)
@@ -263,13 +269,14 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
         {
           if (x < (ssize_t) image->columns)
             {
-              p=GetVirtualPixels(image,x,y,image->columns-x,1,exception);
-              q=GetAuthenticPixels(floodplane_image,x,y,image->columns-x,1,
+              p=GetCacheViewVirtualPixels(image_view,x,y,image->columns-x,1,
                 exception);
+              q=GetCacheViewAuthenticPixels(floodplane_view,x,y,
+                image->columns-x,1,exception);
               if ((p == (const PixelPacket *) NULL) ||
                   (q == (PixelPacket *) NULL))
                 break;
-              indexes=GetVirtualIndexQueue(image);
+              indexes=GetCacheViewVirtualIndexQueue(image_view);
               for ( ; x < (ssize_t) image->columns; x++)
               {
                 if (q->opacity == (Quantum) TransparentOpacity)
@@ -281,7 +288,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
                 p++;
                 q++;
               }
-              if (SyncAuthenticPixels(floodplane_image,exception) == MagickFalse)
+              if (SyncCacheViewAuthenticPixels(floodplane_view,exception) == MagickFalse)
                 break;
             }
           PushSegmentStack(y,start,x-1,offset);
@@ -292,12 +299,13 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       x++;
       if (x <= x2)
         {
-          p=GetVirtualPixels(image,x,y,(size_t) (x2-x+1),1,exception);
-          q=GetAuthenticPixels(floodplane_image,x,y,(size_t) (x2-x+1),1,
+          p=GetCacheViewVirtualPixels(image_view,x,y,(size_t) (x2-x+1),1,
+            exception);
+          q=GetCacheViewAuthenticPixels(floodplane_view,x,y,(size_t) (x2-x+1),1,
             exception);
           if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
-          indexes=GetVirtualIndexQueue(image);
+          indexes=GetCacheViewVirtualIndexQueue(image_view);
           for ( ; x <= x2; x++)
           {
             if (q->opacity == (Quantum) TransparentOpacity)
@@ -329,37 +337,41 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     /*
       Tile fill color onto floodplane.
     */
-    p=GetVirtualPixels(floodplane_image,0,y,image->columns,1,exception);
-    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(floodplane_view,0,y,image->columns,1,
+      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetAuthenticIndexQueue(image);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (p->opacity != OpaqueOpacity)
+      if (GetPixelOpacity(p) != OpaqueOpacity)
         {
           (void) GetFillColor(draw_info,x,y,&fill_color);
           SetMagickPixelPacket(image,&fill_color,(IndexPacket *) NULL,&fill);
           if (image->colorspace == CMYKColorspace)
             ConvertRGBToCMYK(&fill);
           if ((channel & RedChannel) != 0)
-            q->red=ClampToQuantum(fill.red);
+            SetPixelRed(q,ClampToQuantum(fill.red));
           if ((channel & GreenChannel) != 0)
-            q->green=ClampToQuantum(fill.green);
+            SetPixelGreen(q,ClampToQuantum(fill.green));
           if ((channel & BlueChannel) != 0)
-            q->blue=ClampToQuantum(fill.blue);
-          if ((channel & OpacityChannel) != 0)
-            q->opacity=ClampToQuantum(fill.opacity);
+            SetPixelBlue(q,ClampToQuantum(fill.blue));
+          if (((channel & OpacityChannel) != 0) ||
+              (draw_info->fill.opacity != OpaqueOpacity))
+            SetPixelOpacity(q,ClampToQuantum(fill.opacity));
           if (((channel & IndexChannel) != 0) &&
               (image->colorspace == CMYKColorspace))
-            indexes[x]=ClampToQuantum(fill.index);
+            SetPixelIndex(indexes+x,ClampToQuantum(fill.index));
         }
       p++;
       q++;
     }
-    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
   }
+  floodplane_view=DestroyCacheView(floodplane_view);
+  image_view=DestroyCacheView(image_view);
   segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
   floodplane_image=DestroyImage(floodplane_image);
   return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
@@ -403,6 +415,12 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 % This provides a good example of making use of the DrawGradientImage
 % function and the gradient structure in draw_info.
 */
+
+static inline double MagickMax(const double x,const double y)
+{
+  return(x > y ? x : y);
+}
+
 MagickExport MagickBooleanType GradientImage(Image *image,
   const GradientType type,const SpreadMethod method,
   const PixelPacket *start_color,const PixelPacket *stop_color)
@@ -465,12 +483,6 @@ MagickExport MagickBooleanType GradientImage(Image *image,
   */
   status=DrawGradientImage(image,draw_info);
   draw_info=DestroyDrawInfo(draw_info);
-  if ((start_color->opacity == OpaqueOpacity) &&
-      (stop_color->opacity == OpaqueOpacity))
-    image->matte=MagickFalse;
-  if ((IsGrayPixel(start_color) != MagickFalse) &&
-      (IsGrayPixel(stop_color) != MagickFalse))
-    image->type=GrayscaleType;
   return(status);
 }
 
@@ -510,10 +522,10 @@ static size_t **DestroyHistogramThreadSet(size_t **histogram)
     i;
 
   assert(histogram != (size_t **) NULL);
-  for (i=0; i < (ssize_t) GetOpenMPMaximumThreads(); i++)
+  for (i=0; i < (ssize_t) GetMagickResourceLimit(ThreadResource); i++)
     if (histogram[i] != (size_t *) NULL)
       histogram[i]=(size_t *) RelinquishMagickMemory(histogram[i]);
-  histogram=(size_t **) RelinquishAlignedMemory(histogram);
+  histogram=(size_t **) RelinquishMagickMemory(histogram);
   return(histogram);
 }
 
@@ -527,7 +539,7 @@ static size_t **AcquireHistogramThreadSet(const size_t count)
     number_threads;
 
   number_threads=GetOpenMPMaximumThreads();
-  histogram=(size_t **) AcquireAlignedMemory(number_threads,
+  histogram=(size_t **) AcquireQuantumMemory(number_threads,
     sizeof(*histogram));
   if (histogram == (size_t **) NULL)
     return((size_t **) NULL);
@@ -578,8 +590,6 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   width=GetOptimalKernelWidth2D(radius,0.5);
-  if ((image->columns < width) || (image->rows < width))
-    ThrowImageException(OptionError,"ImageSmallerThanRadius");
   paint_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
   if (paint_image == (Image *) NULL)
     return((Image *) NULL);
@@ -600,10 +610,11 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheView(image);
-  paint_view=AcquireCacheView(paint_image);
+  image_view=AcquireVirtualCacheView(image,exception);
+  paint_view=AcquireAuthenticCacheView(paint_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -627,8 +638,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,-((ssize_t) width/2L),y-(ssize_t) (width/
-      2L),image->columns+width,width,exception);
+    p=GetCacheViewVirtualPixels(image_view,-((ssize_t) width/2L),y-(ssize_t)
+      (width/2L),image->columns+width,width,exception);
     q=QueueCacheViewAuthenticPixels(paint_view,0,y,paint_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
@@ -641,11 +652,6 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
     histogram=histograms[GetOpenMPThreadId()];
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      ssize_t
-        j,
-        k,
-        v;
-
       register ssize_t
         i,
         u;
@@ -653,6 +659,11 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       size_t
         count;
 
+      ssize_t
+        j,
+        k,
+        v;
+
       /*
         Assign most frequent color.
       */
@@ -664,7 +675,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       {
         for (u=0; u < (ssize_t) width; u++)
         {
-          k=(ssize_t) ScaleQuantumToChar(PixelIntensityToQuantum(p+u+i));
+          k=(ssize_t) ScaleQuantumToChar(PixelIntensityToQuantum(image,p+u+i));
           histogram[k]++;
           if (histogram[k] > count)
             {
@@ -676,7 +687,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       }
       *q=(*(p+j));
       if (image->colorspace == CMYKColorspace)
-        paint_indexes[x]=indexes[x+j];
+        SetPixelIndex(paint_indexes+x,GetPixelIndex(
+          indexes+x+j));
       p++;
       q++;
     }
@@ -688,7 +700,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_OilPaintImage)
+        #pragma omp critical (MagickCore_OilPaintImage)
 #endif
         proceed=SetImageProgress(image,OilPaintImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
@@ -750,7 +762,7 @@ MagickExport MagickBooleanType OpaquePaintImage(Image *image,
   const MagickPixelPacket *target,const MagickPixelPacket *fill,
   const MagickBooleanType invert)
 {
-  return(OpaquePaintImageChannel(image,AllChannels,target,fill,invert));
+  return(OpaquePaintImageChannel(image,CompositeChannels,target,fill,invert));
 }
 
 MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
@@ -785,6 +797,11 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
+  if ((IsGrayColorspace(image->colorspace) != MagickFalse) &&
+      (IsMagickGray(fill) != MagickFalse))
+    (void) TransformImageColorspace(image,sRGBColorspace);
+  if ((fill->opacity != OpaqueOpacity) && (image->matte == MagickFalse))
+    (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   /*
     Make image color opaque.
   */
@@ -792,9 +809,10 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
   progress=0;
   exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheView(image);
+  image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -826,16 +844,16 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
       if (IsMagickColorSimilar(&pixel,target) != invert)
         {
           if ((channel & RedChannel) != 0)
-            q->red=ClampToQuantum(fill->red);
+            SetPixelRed(q,ClampToQuantum(fill->red));
           if ((channel & GreenChannel) != 0)
-            q->green=ClampToQuantum(fill->green);
+            SetPixelGreen(q,ClampToQuantum(fill->green));
           if ((channel & BlueChannel) != 0)
-            q->blue=ClampToQuantum(fill->blue);
+            SetPixelBlue(q,ClampToQuantum(fill->blue));
           if ((channel & OpacityChannel) != 0)
-            q->opacity=ClampToQuantum(fill->opacity);
+            SetPixelOpacity(q,ClampToQuantum(fill->opacity));
           if (((channel & IndexChannel) != 0) &&
               (image->colorspace == CMYKColorspace))
-            indexes[x]=ClampToQuantum(fill->index);
+            SetPixelIndex(indexes+x,ClampToQuantum(fill->index));
         }
       q++;
     }
@@ -847,7 +865,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_OpaquePaintImageChannel)
+        #pragma omp critical (MagickCore_OpaquePaintImageChannel)
 #endif
         proceed=SetImageProgress(image,OpaquePaintImageTag,progress++,
           image->rows);
@@ -936,9 +954,10 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
   progress=0;
   exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheView(image);
+  image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -979,7 +998,7 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_TransparentPaintImage)
+        #pragma omp critical (MagickCore_TransparentPaintImage)
 #endif
         proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
           image->rows);
@@ -1068,9 +1087,10 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
-  image_view=AcquireCacheView(image);
+  image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -1118,7 +1138,7 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_TransparentPaintImageChroma)
+        #pragma omp critical (MagickCore_TransparentPaintImageChroma)
 #endif
         proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
           image->rows);
